#!/usr/bin/python

__pychecker__ = 'unusednames=node_id'

import unittest

import netsim_dpor


class TestVectorClockOperations(unittest.TestCase):
    def testMerge(self):
        v1 = [0, 1, 2]
        v2 = [0, 0, 3]
        self.assertEquals((0, 1, 3), netsim_dpor.vectorClockMerge(v1, v2))
        self.assertRaises(AssertionError, netsim_dpor.vectorClockMerge, [0], [0, 1])

    def testComparison(self):
        v1 = [0, 1, 2]
        v2 = [0, 1, 2]

        # Concurrent
        self.assertEquals(0, netsim_dpor.vectorClockCompare(v1, v2))
        v1[1] = 0
        v2[2] = 0
        self.assertEquals(0, netsim_dpor.vectorClockCompare(v1, v2))

        # v1 happens-before v2
        v1 = [0, 1, 0]
        v2 = [0, 1, 2]
        self.assertEquals(-1, netsim_dpor.vectorClockCompare(v1, v2))
        self.assertEquals(1, netsim_dpor.vectorClockCompare(v2, v1))


class MockStateMachine(object):
    def __init__(self, start_destination):
        self._start = start_destination

    def messageReceived(self, system, source, payload):
        self.system = system
        self.source = source
        self.payload = payload

    def start(self, system, node_id):
        if self._start is not None:
            system.sendMessage(self._start, "foo")


class MockSystem(object):
    pass


class NodeStateTest(unittest.TestCase):
    def setUp(self):
        self.system = MockSystem()
        self.machine = MockStateMachine(None)
        self.state = netsim_dpor.NodeState(self.machine)
        self.state.start(self.system, 0, 4)

        self.initial_history = self.state.historyHash()
        self.message = netsim_dpor.Message("id", 0, 1, "data", [0, 1, 0, 0])

    def testVectorClock(self):
        self.state = netsim_dpor.NodeState(self.machine)
        self.state.start(self.system, 0, 4)
        self.assertEquals([1, 0, 0, 0], self.state.vector_clock)

        self.state.messageReceived(self.system, self.message)
        self.assertEquals([2, 1, 0, 0], self.state.vector_clock)
        self.assertEquals((3, 1, 0, 0), self.state.getSentVectorClock())
        self.assertEquals([3, 1, 0, 0], self.state.vector_clock)

        self.state.messageReceived(self.system, self.message)
        self.assertEquals([4, 1, 0, 0], self.state.vector_clock)


if __name__ == "__main__":
    unittest.main()
